"""Model implements CMS-HCC software V2318.83.P1"""

from pyriskadjust.icd_mapping.mapping_2019_v23 import ICD_MAPPING
from pyriskadjust.hccs.hccs_v23 import HCC_HIERARCHY
from pyriskadjust.hccs.hccs_v23 import HCC_LABELS
from pyriskadjust.coefficients.coefficients_2019_v23 import COEFFICIENTS
from pyriskadjust.models.common import (
    MODEL_DESCRIPTIONS,
    MODEL_ABBREVIATIONS,
    get_age_sex_string,
    _diagnoses_to_hccs,
    _explain_score,
)
import logging

INTERACTION_VARIABLE_DESCRIPTIONS = {
    "hcc47_gcancer": "Immunity Disorders & Cancer",
    "hcc85_gdiabetesmellit": "Congestive Heart Failure & Diabetes",
    "hcc85_gcopdcf": "Congestive Heart Failure & Cystic Fibrosis/COPD",
    "hcc85_grenal_v23": "Congestive Heart Failure & Renal Failure (2019)",
    "grespdepandarre_gcopdcf": "Cardio-Respiratory Failure & Cystic Fibrosis/COPD",
    "hcc85_hcc96": "Congestive Heart failure & Specified Heart Arrhythmias",
    "disable_substabuse_psych_v23": "Substance Misuse & Psychiatric Disorder (2019)",
    "originallydisabled_female": "Female who originally qualified due to disability",
    "originallydisabled_male": "Male who originally qualified due to disability",
    "chf_gcopdcf": "Congestive Heart Failure & Cystic Fibrosis/COPD",
    "gcopdcf_card_resp_fail": "Cardio-Respiratory Failure & Cystic Fibrosis/COPD",
    "sepsis_pressure_ulcer": "Sepsis & Pressure Ulcer",
    "sepsis_artif_openings": "Sepis & Angina Pectoris",
    "art_openings_press_ulcer": "Pressure Ulcer & Angina Pectoris",
    "diabetes_chf": "Diabetes & Congestive Heart Failure",
    "gcopdcf_asp_spec_bact_pneum": "COPD & Aspiration and Specified Bacterial Pneumonias",
    "asp_spec_b_pneum_pres_ulc": "Pressure Ulcer & Aspiration and Specified Bacterial Pneumonias",
    "sepsis_asp_spec_bact_pneum": "Sepsis & Aspiration and Specified Bacterial Pneumonias",
    "schizophrenia_gcopdcf": "Schizophrenia & Cystic Fibrosis/COPD",
    "schizophrenia_chf": "Schizophrenia & Congestive Heart Failure",
    "schizophrenia_seizures": "Schizophrenia & Seizures",
    "disabled_hcc85": "Disabled & Congestive Heart Failure",
    "disabled_pressure_ulcer": "Disabled & Pressure Ulcer",
    "disabled_hcc161": "Disabled & Chronic Ulcer of Skin, Except Pressure",
    "disabled_hcc39": "Disabled & Bone/Joint/Muscle Infections/Necrosis",
    "disabled_hcc77": "Disabled & Multiple Sclerosis",
    "disabled_hcc6": "Disbaled & Opportunistic Infections",
    "ltimcaid": "Institutional Model & Patient on Medicaid at least part of the payment year",
    "origds": "Patient is over 65 & Original reason for entitlement is disability",
}


def explain_score(score_components):
    return _explain_score(
        MODEL_ABBREVIATIONS,
        INTERACTION_VARIABLE_DESCRIPTIONS,
        HCC_LABELS,
        score_components,
    )


def diagnoses_to_hccs(diagnoses, age, sex):
    return _diagnoses_to_hccs(ICD_MAPPING, HCC_HIERARCHY, diagnoses, age, sex)


def compute_risk_score_components(
    diagnoses,
    age,
    sex,
    long_term_institutional_in_medicaid=False,
    new_enrollee_in_medicaid=False,
    original_entitlement_reason=0,
    model="cna",
):
    """Computes the risk score for a patient, given a list of diagnoses as ICD_10 codes,
    their age, sex, etc. 

    Arguments:
        diagnoses {[list]} -- List of ICD-10 codes, as strings
        age {int} -- patient's age, as integer. This should be computed as of Feb 1 for a given model year
        sex {int} -- 1=male, 2=female

    Keyword Arguments:
        long_term_institutional_in_medicaid {bool} -- True if number of months in Medicaid in payment year > 0. This is only relevant to the Institutional model (default: {False})
        new_enrollee_in_medicaid {bool} -- True if new Medicare enrollee and number of months in Medicaid in payment year > 0. This is only relevant to the two New Enrollee models (default: {False})
        original_entitlement_reason {int} -- Original entitlement reason. 0 = Old Age, 1 = Disability, 2 = End Stage Renal Disease, 3 = both Disability and ESRD (default: {0})
        model {str} -- Abbreviation for the model to use (default: {"cna"})

    Returns:
        dict -- Dictionary of the form 
        {
            "coefficient_name" : coefficient_value,
            "coefficient_name" : coefficient_value
        }
    """

    output = {}

    model_prefix = "{}_".format(model)

    # young and disabled
    is_disabled = age < 65 and int(original_entitlement_reason) != 0
    # old but original entitlement reason is disability
    is_originally_disabled = age >= 65 and int(original_entitlement_reason) == 1

    # --------- New Enrollee and SNP New Enrollee models -----

    # These models are not based on HCCs. They are based solely on sex, age,
    # whether the new enrollee qualified due to disability, and
    # whether the new enrollee was on medicaid for at least part of the year
    if model == "ne" or model == "snpne":
        if not new_enrollee_in_medicaid and not is_originally_disabled:
            demographic_var = model_prefix + "nmcaid_norigdis_"
        elif new_enrollee_in_medicaid and not is_originally_disabled:
            demographic_var = model_prefix + "mcaid_norigdis_"
        elif not new_enrollee_in_medicaid and is_originally_disabled:
            demographic_var = model_prefix + "nmcaid_origdis_"
        elif new_enrollee_in_medicaid and is_originally_disabled:
            demographic_var = model_prefix + "mcaid_origdis_"

        demographic_var += get_age_sex_string(age, sex, new_enrollee=True)
        if demographic_var in COEFFICIENTS:
            output[demographic_var] = COEFFICIENTS[demographic_var]
        else:
            logging.warning(
                "Demographic coefficient not found for patient with age {} and sex {}: {}".format(
                    age, sex, demographic_var
                )
            )
        return output

    # --------- For all other models -----------------------------------

    # Start by getting the demographic variable based on age and sex
    demographic_var = model_prefix + get_age_sex_string(age, sex, new_enrollee=False)
    if demographic_var in COEFFICIENTS:
        output[demographic_var] = COEFFICIENTS[demographic_var]
    else:
        logging.warning(
            "Demographic coefficient not found for patient with age {} and sex {}: {}".format(
                age, sex, demographic_var
            )
        )

    # Now compute the relevant HCCs
    hccs = diagnoses_to_hccs(diagnoses, age, sex)
    hcc_vars = [model_prefix + "hcc{}".format(hcc) for hcc in hccs]
    for v in hcc_vars:
        if v in COEFFICIENTS:
            output[v] = COEFFICIENTS[v]
        else:
            logging.warning("HCC coefficient not found: {}".format(v))

    # Now compute the interaction components
    interaction_vars = []

    # common variables to compute interaction vars
    cancer = bool(hccs.intersection({8, 9, 10, 11, 12}))
    diabetes = bool(hccs.intersection({17, 18, 19}))
    card_resp_fail = bool(hccs.intersection({82, 83, 84}))
    chf = 85 in hccs
    gCopdCF = bool(hccs.intersection({110, 111, 112}))
    renal_v23 = bool(hccs.intersection({134, 135, 136, 137, 138}))
    sepsis = 2 in hccs
    gSubstanceAbuse_v23 = bool(hccs.intersection({54, 55, 56}))
    gPsychiatric_v23 = bool(hccs.intersection({57, 58, 59, 60}))

    # --------- Community Models ------------------

    if model in {"cna", "cnd", "cfa", "cfd", "cpa", "cpd"}:
        # %*community models interactions
        if 47 in hccs and cancer:
            interaction_vars.append("hcc47_gcancer")
        if chf and diabetes:
            interaction_vars.append("hcc85_gdiabetesmellit")
        if chf and gCopdCF:
            interaction_vars.append("hcc85_gcopdcf")
        if chf and renal_v23:
            interaction_vars.append("hcc85_grenal_v23")
        if card_resp_fail and gCopdCF:
            interaction_vars.append("grespdepandarre_gcopdcf")
        if chf and 96 in hccs:
            interaction_vars.append("hcc85_hcc96")

        # in the models for aged patients, we also take into account
        # whether they originally qualified due to disability
        if model in {"cna", "cfa", "cpa"}:
            if is_originally_disabled and int(sex) == 2:
                interaction_vars.append("originallydisabled_female")
            if is_originally_disabled and int(sex) == 1:
                interaction_vars.append("originallydisabled_male")

        if model in {"cnd", "cfd", "cpd"}:
            if age < 65 and gSubstanceAbuse_v23 and gPsychiatric_v23:
                interaction_vars.append("disable_substabuse_psych_v23")

    # --------- Institutional Model ------------------

    if model == "ins":
        # %*institutional model;
        pressure_ulcer = bool(hccs.intersection({157, 158}))  # /*10/19/2012*/

        if chf and gCopdCF:
            interaction_vars.append("chf_gcopdcf")
        if gCopdCF and card_resp_fail:
            interaction_vars.append("gcopdcf_card_resp_fail")
        if sepsis and pressure_ulcer:
            interaction_vars.append("sepsis_pressure_ulcer")
        if sepsis and 188 in hccs:
            interaction_vars.append("sepsis_artif_openings")
        if pressure_ulcer and 188 in hccs:
            interaction_vars.append("art_openings_press_ulcer")

        if diabetes and chf:
            interaction_vars.append("diabetes_chf")
        if gCopdCF and 114 in hccs:
            interaction_vars.append("gcopdcf_asp_spec_b_pneum")
        if pressure_ulcer and 114 in hccs:
            interaction_vars.append("asp_spec_b_pneum_pres_ulc")
        if sepsis and 114 in hccs:
            interaction_vars.append("sepsis_asp_spec_bact_pneum")
        if gCopdCF and 57 in hccs:
            interaction_vars.append("schizophrenia_gcopdcf")
        if chf and 57 in hccs:
            interaction_vars.append("schizophrenia_chf")
        if 57 in hccs and 79 in hccs:
            interaction_vars.append("schizophrenia_seizures")

        if is_disabled and 85 in hccs:
            interaction_vars.append("disabled_hcc85")
        if is_disabled and pressure_ulcer:
            interaction_vars.append("disabled_pressure_ulcer")
        if is_disabled and 161 in hccs:
            interaction_vars.append("disabled_hcc161")
        if is_disabled and 39 in hccs:
            interaction_vars.append("disabled_hcc39")
        if is_disabled and 77 in hccs:
            interaction_vars.append("disabled_hcc77")
        if is_disabled and 6 in hccs:
            interaction_vars.append("disabled_hcc6")

        if long_term_institutional_in_medicaid:
            interaction_vars.append("ltimcaid")
        if is_originally_disabled:
            interaction_vars.append("origds")

    for v in (model_prefix + iv for iv in interaction_vars):
        if v in COEFFICIENTS:
            output[v] = COEFFICIENTS[v]
        else:
            logging.warning("Warning, interaction coefficient not found: {}".format(v))

    return output



def transitive_reduction(ancestor_dict):
    output = {k: v.copy() for (k, v) in ancestor_dict.items()}
    for i, ancestors in output.items():
        for middle in set(ancestors):
            for anc in set(ancestors) - set({middle}):
                if anc in output.get(middle, []):
                    output[i].remove(anc)
    return output



from datetime import datetime
import re


MODEL_DESCRIPTIONS = {
    "cna": "Community Non-Dual Aged",
    "cnd": "Community Non-Dual Disabled",
    "cfa": "Community Full Benefit Dual Aged",
    "cfd": "Community Full Benefit Dual Disabled",
    "cpa": "Community Partial Benefit Dual Aged",
    "cpd": "Community Partial Benefit Dual Disabled",
    "ins": "Long Term Institutional",
    "ne": "New Enrollees",
    "snpne": "Special Needs Plan (SNP) New Enrollees",
}

MODEL_ABBREVIATIONS = MODEL_DESCRIPTIONS.keys()


def get_age_sex_string(age, sex, new_enrollee=False):
    age_sex_string = "m" if int(sex) == 1 else "f"
    if 0 <= age <= 34:
        return age_sex_string + "0_34"
    elif 35 <= age <= 44:
        return age_sex_string + "35_44"
    elif 45 <= age <= 54:
        return age_sex_string + "45_54"
    elif 55 <= age <= 94:
        if new_enrollee and 65 <= age <= 69:
            return age_sex_string + str(age)
        else:
            lower = int(age / 5) * 5
            upper = lower + 4
            return age_sex_string + "{}_{}".format(lower, upper)
    elif 95 <= age:
        return age_sex_string + "95_gt"


def _diagnoses_to_hccs(icd_mapping, hcc_hierachy, diagnoses, age, sex):
    """Returns a list of hierachical condition categories, implied by a set of diagnoses

    Arguments:
        diagnoses {[string]} -- A list of ICD-10 codes

    Returns:
        [int] -- A list of HCCs, represented as ints
    """
    # Normalize codes by uppercasing and stripping out periods
    diagnoses = [d.strip().upper().replace(".", "") for d in diagnoses]

    hccs = set()

    # get the union of all hccs implied by individual diagnoses
    for d in diagnoses:
        # some special case edits based on V22I0ED2.TXT
        if sex == 2 and d in {"D66", "D67"}:
            hccs.update({48})
        elif age < 18 and d in {
            "J410",
            "J411",
            "J418",
            "J42",
            "J430",
            "J431",
            "J432",
            "J438",
            "J439",
            "J440",
            "J441",
            "J449",
            "J982",
            "J983",
        }:
            hccs.update({112})
        elif age < 6 or age > 18 and d == "F3481":
            pass
        else:
            # If not special case, default to general mapping
            hccs.update(icd_mapping.get(d, []))

    # remove HCCs that are already implied by more specific categories in the hierachy
    for cc in hccs.copy():
        hccs.difference_update(hcc_hierachy.get(cc, []))

    return hccs


def get_age_in_model_year(dob, model_year):
    """Gets the age of a patient relative to Feb 1 of a given model year

    Arguments:
        dob {datetime.datetime | string} -- dob as a datetime or string in format 'YYYY-MM-DD'
        model_year {int} -- Year as

    Returns:
        int -- age as an integer
    """

    if type(dob) == str:
        dob = datetime.strptime(dob, "%Y-%m-%d")
    as_of_date = datetime(year=model_year, month=2, day=1)
    return (
        as_of_date.year
        - dob.year
        - ((as_of_date.month, as_of_date.day) < (dob.month, dob.day))
    )


def _explain_score(
    model_abbreviations, interaction_var_descriptions, hcc_labels, score_components
):
    """Given a dictionay of model coefficients and scores, returns
    a more verbose explanation of what the coefficients mean, and
    how the individual components contribute to the overall score

    Arguments:
        score_components {dict} -- A dictionary of coefficients of the form
            {"coefficient_name" : coefficient_value}
        Intended to be called on the result of compute_risk_score_componenents

    Returns:
        dict -- A dictionary with more info, of the form
        {
             "total": number,
            "demographic_components": [{
                "variable_name" : name of the coefficient,
                "description" : human readable description of what the coefficient means,
                "score" : magnitude of the coefficient
            }],
            "hcc_components": [{
                "variable_name" : name of the coefficient,
                "description" : human readable description of what the coefficient means,
                "score" : magnitude of the coefficient
            }],
            "interaction_components": [{
                "variable_name" : name of the coefficient,
                "description" : human readable description of what the coefficient means,
                "score" : magnitude of the coefficient
            }],
        }
    """
    output = {
        "total": round(sum(score_components.values()), 3),
        "demographic_components": [],
        "hcc_components": [],
        "interaction_components": [],
    }
    # output["normalized_total"] = output["total"] / CODING_INTENSITY_NORMALIZATION

    demographic_regex = re.compile(
        """
        \S*                     # any chars
        (?P<sex>m|f)            # sex
        (?P<age_lo>\d{1,2})     # lower age limit
        (_                      # optional underscore
        (?P<age_hi>\d{1,2}|gt)  # optional upper age limit
        )?$
        """,
        re.VERBOSE,
    )
    interaction_regex = re.compile(
        """
        \S*                     # any
        (?P<var_name>{})$       # name of interaction variables
        """.format(
            "|".join(interaction_var_descriptions.keys())
        ),
        re.VERBOSE,
    )
    hcc_regex = re.compile(
        """
        ^(?P<model_abbr>{})      # model abbreviation
        \_hcc
        (?P<hcc>\d+)$           # hcc number
        """.format(
            "|".join(model_abbreviations)
        ),
        re.VERBOSE,
    )
    for component, score in score_components.items():
        m = re.match(demographic_regex, component)
        if m:
            sex = m.group("sex")
            age_lo = m.group("age_lo")
            age_hi = m.group("age_hi")
            description = "{} with age ".format("Male" if sex == "m" else "Female")
            if age_hi == "gt":
                description += " greater than {}".format(age_lo)
            elif age_hi:
                description += "in range {} to {}".format(age_lo, age_hi)
            else:
                description += "equal to {}".format(age_lo)

            output["demographic_components"].append(
                {"variable_name": component, "score": score, "description": description}
            )

        m = re.match(interaction_regex, component)
        if m:
            output["interaction_components"].append(
                {
                    "variable_name": component,
                    "score": score,
                    "description": interaction_var_descriptions.get(
                        m.group("var_name"), "unknown"
                    ),
                }
            )

        m = re.match(hcc_regex, component)
        if m:
            output["hcc_components"].append(
                {
                    "variable_name": component,
                    "score": score,
                    "description": hcc_labels[int(m.group("hcc"))],
                }
            )
    return output




from pyriskadjust.icd_mapping.mapping_2019_v23 import ICD_MAPPING
from pyriskadjust.hccs.hccs_v23 import HCC_LABELS

import json

output = dict((n, []) for n in HCC_LABELS.keys())
for icd, hccs in ICD_MAPPING.items():
    for hcc in hccs:
        output[hcc].append(icd)

with open("hcc_to_icd_2018.json", "w") as f:
    json.dump(output, f, indent=4)



#%***********************************************************************
# The labels and hierarchical relationships between HCC categories in
# model version 23
#
# Based on files: V23H83L2.TXT (labels) and V23H83H2.TXT (hierarchy)
# ***********************************************************************;

HCC_LABELS = {
    1: "HIV/AIDS",
    2: "Septicemia, Sepsis, Systemic Inflammatory Response Syndrome/Shock",
    6: "Opportunistic Infections",
    8: "Metastatic Cancer and Acute Leukemia",
    9: "Lung and Other Severe Cancers",
    10: "Lymphoma and Other Cancers",
    11: "Colorectal, Bladder, and Other Cancers",
    12: "Breast, Prostate, and Other Cancers and Tumors",
    17: "Diabetes with Acute Complications",
    18: "Diabetes with Chronic Complications",
    19: "Diabetes without Complication",
    21: "Protein-Calorie Malnutrition",
    22: "Morbid Obesity",
    23: "Other Significant Endocrine and Metabolic Disorders",
    27: "End-Stage Liver Disease",
    28: "Cirrhosis of Liver",
    29: "Chronic Hepatitis",
    33: "Intestinal Obstruction/Perforation",
    34: "Chronic Pancreatitis",
    35: "Inflammatory Bowel Disease",
    39: "Bone/Joint/Muscle Infections/Necrosis",
    40: "Rheumatoid Arthritis and Inflammatory Connective Tissue Disease",
    46: "Severe Hematological Disorders",
    47: "Disorders of Immunity",
    48: "Coagulation Defects and Other Specified Hematological Disorders",
    54: "Substance Use with Psychotic Complications",
    55: "Substance Use Disorder, Moderate/Severe, or Substance Use with Complications",
    56: "Substance Use Disorder, Mild, Except Alcohol and Cannabis",
    57: "Schizophrenia",
    58: "Reactive and Unspecified Psychosis",
    59: "Major Depressive, Bipolar, and Paranoid Disorders",
    60: "Personality Disorders",
    70: "Quadriplegia",
    71: "Paraplegia",
    72: "Spinal Cord Disorders/Injuries",
    73: "Amyotrophic Lateral Sclerosis and Other Motor Neuron Disease",
    74: "Cerebral Palsy",
    75: "Myasthenia Gravis/Myoneural Disorders and Guillain-Barre Syndrome/Inflammatory and Toxic Neuropathy",
    76: "Muscular Dystrophy",
    77: "Multiple Sclerosis",
    78: "Parkinson's and Huntington's Diseases",
    79: "Seizure Disorders and Convulsions",
    80: "Coma, Brain Compression/Anoxic Damage",
    82: "Respirator Dependence/Tracheostomy Status",
    83: "Respiratory Arrest",
    84: "Cardio-Respiratory Failure and Shock",
    85: "Congestive Heart Failure",
    86: "Acute Myocardial Infarction",
    87: "Unstable Angina and Other Acute Ischemic Heart Disease",
    88: "Angina Pectoris",
    96: "Specified Heart Arrhythmias",
    99: "Intracranial Hemorrhage",
    100: "Ischemic or Unspecified Stroke",
    103: "Hemiplegia/Hemiparesis",
    104: "Monoplegia, Other Paralytic Syndromes",
    106: "Atherosclerosis of the Extremities with Ulceration or Gangrene",
    107: "Vascular Disease with Complications",
    108: "Vascular Disease",
    110: "Cystic Fibrosis",
    111: "Chronic Obstructive Pulmonary Disease",
    112: "Fibrosis of Lung and Other Chronic Lung Disorders",
    114: "Aspiration and Specified Bacterial Pneumonias",
    115: "Pneumococcal Pneumonia, Empyema, Lung Abscess",
    122: "Proliferative Diabetic Retinopathy and Vitreous Hemorrhage",
    124: "Exudative Macular Degeneration",
    134: "Dialysis Status",
    135: "Acute Renal Failure",
    136: "Chronic Kidney Disease, Stage 5",
    137: "Chronic Kidney Disease, Severe (Stage 4)",
    138: "Chronic Kidney Disease, Moderate (Stage 3)",
    157: "Pressure Ulcer of Skin with Necrosis Through to Muscle, Tendon, or Bone",
    158: "Pressure Ulcer of Skin with Full Thickness Skin Loss",
    161: "Chronic Ulcer of Skin, Except Pressure",
    162: "Severe Skin Burn or Condition",
    166: "Severe Head Injury",
    167: "Major Head Injury",
    169: "Vertebral Fractures without Spinal Cord Injury",
    170: "Hip Fracture/Dislocation",
    173: "Traumatic Amputations and Complications",
    176: "Complications of Specified Implanted Device or Graft",
    186: "Major Organ Transplant or Replacement Status",
    188: "Artificial Openings for Feeding or Elimination",
    189: "Amputation Status, Lower Limb/Amputation Complications",
}

# Map HCC category to parents (more general categories)
HCC_HIERARCHY = {
    8: [9, 10, 11, 12],  # /*Neoplasm 1 */
    9: [10, 11, 12],  # /*Neoplasm 2 */
    10: [11, 12],  # /*Neoplasm 3 */
    11: [12],  # /*Neoplasm 4 */
    17: [18, 19],  # /*Diabetes 1 */
    18: [19],  # /*Diabetes 2 */
    27: [28, 29, 80],  # /*Liver 1 */
    28: [29],  # /*Liver 2 */
    46: [48],  # /*Blood 1 */
    54: [55, 56],  # /*SA 1 */
    55: [56],  # /*SA 2 */
    57: [58, 59, 60],  # /*Psychiatric 1 */
    58: [59, 60],  # /*Psychiatric 2 */
    59: [60],  # /*Psychiatric 3 */
    70: [71, 72, 103, 104, 169],  # /*Spinal 1 */
    71: [72, 104, 169],  # /*Spinal 2 */
    72: [169],  # /*Spinal 3 */
    82: [83, 84],  # /*Arrest 1 */
    83: [84],  # /*Arrest 2 */
    86: [87, 88],  # /*Heart 2 */
    87: [88],  # /*Heart 3 */
    99: [100],  # /*CVD 1 */
    103: [104],  # /*CVD 5 */
    106: [107, 108, 161, 189],  # /*Vascular 1 */
    107: [108],  # /*Vascular 2 */
    110: [111, 112],  # /*Lung 1 */
    111: [112],  # /*Lung 2 */
    114: [115],  # /*Lung 5 */
    134: [135, 136, 137, 138],  # /*Kidney 3 */
    135: [136, 137, 138],  # /*Kidney 4 */
    136: [137, 138],  # /*Kidney 5 */
    137: [138],  # /*Kidney 6 */
    157: [158, 161],  # /*Skin 1 */
    158: [161],  # /*Skin 2 */
    166: [80, 167],  # /*Injury 1 */
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pyriskadjust` package."""


import unittest
import json
from pyriskadjust.models import model_2018_v22


class TestPyriskadjust(unittest.TestCase):
    """Tests for `pyriskadjust` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_model_community_aged_nondual(self):
        self.assertEqual(
            json.dumps(
                model_2018_v22.compute_risk_score_components(
                    ["E1169", "I5030", "I509", "I211", "I209", "R05"], age=70, sex=1
                ),
                sort_keys=True,
                indent=2,
            ),
            json.dumps(
                {
                    "cna_m70_74": 0.379,
                    "cna_hcc88": 0.14,
                    "cna_hcc18": 0.318,
                    "cna_hcc85": 0.323,
                    "cna_hcc85_gdiabetesmellit": 0.154,
                },
                sort_keys=True,
                indent=2,
            ),
        )

    def test_explain_total(self):
        explanation = model_2018_v22.explain_score(
            {
                "cna_m70_74": 0.379,
                "cna_hcc88": 0.14,
                "cna_hcc18": 0.318,
                "cna_hcc85": 0.323,
                "cna_hcc85_gdiabetesmellit": 0.154,
            }
        )
        self.assertEqual(explanation["total"], 1.314)
        self.assertEqual(
            json.dumps(explanation["demographic_components"][0], sort_keys=True),
            json.dumps(
                {
                    "variable_name": "cna_m70_74",
                    "score": 0.379,
                    "description": "Male with age in range 70 to 74",
                },
                sort_keys=True,
            ),
        )
        self.assertEqual(
            json.dumps(explanation["interaction_components"][0], sort_keys=True),
            json.dumps(
                {
                    "variable_name": "cna_hcc85_gdiabetesmellit",
                    "score": 0.154,
                    "description": "Congestive Heart Failure & Diabetes",
                },
                sort_keys=True,
            ),
        )
